#############
# $Id: touchSplitFiles.pm 13203 2011-05-18 10:56:49Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2010-09-02
# Last Changed On: $Date: 2011-05-18 11:56:49 +0100 (Wed, 18 May 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.0/lib/npg_pipeline/archive/file/generation/touchSplitFiles.pm $

package npg_pipeline::archive::file::generation::touchSplitFiles;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly;

extends q{npg_pipeline::archive::file::generation};

with qw{npg_common::roles::run::lane::tag_info};

Readonly::Scalar our $VERSION  => do { my ($r) = q$LastChangedRevision: 13203 $ =~ /(\d+)/mxs; $r; };

=head1 NAME

npg_pipeline::archive::file::generation::touchSplitFiles

=head1 VERSION

$LastChangedRevision: 13203 $

=head1 SYNOPSIS

  my $oAfgfq = npg_pipeline::archive::file::generation::touchSplitFile->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module which will go an touch any fastq files which should be found for a lane which contains
a multiplex pool of samples. This ensures that any processes which act on the presence of those files
will find them, even if no sequences for that tag were actually found

=head1 SUBROUTINES/METHODS

=head2 generate

Goes and touches any fastq files which could reasonably be expected to be created for a particular tag
The tags are based on a lane_X.tag file being present in the runfolder directory

Returns an empty array if all is OK

=cut

sub generate {
  my ( $self, $arg_refs ) = @_;

  my $archive_path = $self->archive_path();

  if ( ! -d $archive_path ) {
    my $return = qx{mkdir -p $archive_path};
    if ( $CHILD_ERROR ) {
      croak $archive_path . qq{ does not exist and unable to create: $CHILD_ERROR\n$return};
    }
  }

  my @positions = $self->positions( $arg_refs );

  my $href = $self->lanes_by_split_nonsplit( @positions );


  my @mp_lanes;

  foreach my $lane ( @{ $href->{split_nc_mp_lanes} } ) {
    push @mp_lanes, $lane;
  }

  foreach my $lane ( @{ $href->{non_split_nc_mp_lanes} } ) {
    push @mp_lanes, $lane;
  }

  if ( $self->verbose() ) {
    my $lanes_string = join q{,}, @mp_lanes;
    $self->log( q{mp_lanes } . $lanes_string );
  }

  foreach my $lane ( @mp_lanes ) {

    my $lane_dir = $archive_path . q{/lane} . $lane;
    if ( ! -d $lane_dir ) {
      my $return = qx{mkdir -p $lane_dir};
      if ( $CHILD_ERROR ) {
        croak $archive_path . qq{ does not exist and unable to create: $CHILD_ERROR\n$return};
      }
    }

    my $temp = $self->get_fastq_file_by_tag( $lane );

    foreach my $key ( keys %{ $temp } ) {

      foreach my $index ( keys %{ $temp->{ $key } } ) {

        foreach my $fn ( @{ $temp->{ $key }->{ $index } } ) {

          my $cmd = qq{touch $fn};

          if ( $self->verbose() ) {
            $self->log( $cmd );
          }

          my $return = qx{$cmd};
          if ( $CHILD_ERROR ) {
            croak qq{unable to touch\n\t} . $fn . q{:} . $CHILD_ERROR . qq{\n$return};
          }

        }

      }

    }

  }

  return ();
}


no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive::file::generation

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
