#############
# $Id: split_nonconsented.pm 13816 2011-08-03 11:02:01Z dj3 $
# Created By: ajb
# Last Maintained By: $Author: dj3 $
# Created On: 2009-07-16
# Last Changed On: $Date: 2011-08-03 12:02:01 +0100 (Wed, 03 Aug 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.0/lib/npg_pipeline/archive/file/split_nonconsented.pm $

package npg_pipeline::archive::file::split_nonconsented;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13816 $ =~ /(\d+)/mxs; $r; };

extends q{npg_pipeline::archive::file::generation};

with qw{
  npg_pipeline::roles::business::pre_exec_strings
};

# leaving these two here - should be noted that this is found to be the most optimal
# way of running, so may need changing
Readonly::Scalar our $DEFAULT_NUMBER_OF_PROCESSORS => 8;
Readonly::Scalar our $DEFAULT_RESOURCES => q{-R 'select[mem>8000] rusage[mem=8000] span[hosts=1]' -M8000000 -n} . $DEFAULT_NUMBER_OF_PROCESSORS;

sub split_run_lanes {
  my ($self, $arg_refs, $no_repeat_splitting) = @_;

  my @lanes;
  my $run = $self->run();
  if ( $self->is_paired_read() ) {
    $arg_refs->{paired}++;
  }
  if ( $self->is_indexed() ) {
    $arg_refs->{multiplexed}++;
  }

  @lanes = $self->positions($arg_refs);

  my @wanted_lanes;

  foreach my $lane ( @lanes ) {
    if ( $self->_does_lane_need_splitting($lane) eq q{true} &&
            (!$no_repeat_splitting || $self->_not_split($lane))) {
      push @wanted_lanes, $lane;
    }
  }

  # get out if no lanes require splitting
  if ( ! scalar @wanted_lanes ) {
    return ();
  }

  $arg_refs->{array_string} = $self->create_array_string( @wanted_lanes );

  my $lane = $self->lsb_jobindex();

  my $job_id = $self->submit_bsub_command( $self->_generate_bsub_command( $lane, $arg_refs ) );

  return ( $job_id );
}

### private methods

###############
# checks if split files are already available
sub _not_split {
  my ($self, $lane) = @_;
  my $prefix = $self->archive_path() . q{/} . $self->id_run() . q{_} . $lane;
  if (-e $prefix . q[.fastq]) {
      if (-e $prefix . q[_nonhuman.fastq]) {return 0;}
  } else {
      if (-e $prefix . q[_1.fastq] && -e $prefix . q[_1_nonhuman.fastq]) {return 0;}
  }
  return 1;
}

###############
# responsible for generating the bsub command to be executed
sub _generate_bsub_command {
  my ($self, $lane, $arg_refs) = @_;

  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $self->timestamp();

  my $archive_dir = $self->archive_path();

  my $location_of_logs = $self->make_log_dir( $self->archive_path() );

  my ($fastq1, $fastq2, $fastqt);
  if ($arg_refs->{paired}) {
    $fastq1 = $self->fastq_file_constructs();
    $fastq1 =~ s/substitute_this/substitute_this_1/xms;
    $fastq1 =~ s/substitute_this/$lane/xms;
    $fastq2 = $self->fastq_file_constructs();
    $fastq2 =~ s/substitute_this/substitute_this_2/xms;
    $fastq2 =~ s/substitute_this/$lane/xms;
    if ($arg_refs->{multiplexed}) {
      $fastqt = $self->fastq_file_constructs();
      $fastqt =~ s/substitute_this/substitute_this_t/xms;
      $fastqt =~ s/substitute_this/$lane/xms;
    }
  } else {
    if ($arg_refs->{multiplexed}) {
      $fastq1 = $self->fastq_file_constructs();
      $fastq1 =~ s/substitute_this/substitute_this_1/xms;
      $fastq1 =~ s/substitute_this/$lane/xms;
      $fastqt = $self->fastq_file_constructs();
      $fastqt =~ s/substitute_this/substitute_this_t/xms;
      $fastqt =~ s/substitute_this/$lane/xms;
    } else {
      $fastq1 = $self->fastq_file_constructs();
      $fastq1 =~ s/substitute_this/$lane/xms;
    }
  }

  my $job_name = q{split_nonconsented_} . $self->id_run() . q{_} . $timestamp;
  my $outfile = $location_of_logs . q{/} . $job_name . q{.%I.%J.out};
  $job_name .= $arg_refs->{array_string};

  my $bsub_command = q{bsub -q } . $self->lsf_queue() . qq{ $required_job_completion -J $job_name };
  $bsub_command .=  ( $self->fs_resource_string( {
    resource_string => $DEFAULT_RESOURCES,
    counter_slots_per_job => $DEFAULT_NUMBER_OF_PROCESSORS,
  } ) ) . q{ };
  $bsub_command .=  q{-o } . $outfile . q{ };
  $bsub_command .=  $self->ref_adapter_pre_exec_string() . q{ };
  $bsub_command .=  q{'} . $self->internal_script_bin() . q{/} . $self->external_script_names_conf()->{split_program_file} . q{ };

  $bsub_command .= qq{-fastq1 $archive_dir/$fastq1 };
  if ($arg_refs->{paired}) {
    $bsub_command .= qq{-fastq2 $archive_dir/$fastq2 };
  }
  if ($arg_refs->{multiplexed}) {
    $bsub_command .= qq{-fastqt $archive_dir/$fastqt };
  }
  $bsub_command .=  q{-fork_alignment -id_run } . $self->id_run() . qq{ -position $lane -output_dir $archive_dir'};
  if ($self->verbose()) {
    $self->log( $bsub_command) ;
  }
  return $bsub_command;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::file::split_nonconsented

=head1 VERSION

$LastChangedRevision: 13816 $

=head1 SYNOPSIS

  my $oSplitNC = npg_pipeline::archive::file::split_nonconsented->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module for assessing which lanes require unconsented human to be removed and then bsubs jobs for these

=head1 SUBROUTINES/METHODS

=head2 split_run_lanes - method to call which determines which run lanes require splitting and then submits the jobs to LSF, returning an array of job_ids

  my @job_ids = $oSplitNC->split_run_lanes({
    required_job_completion => $sRequiredJobsToComplete,
    timestamp => $sTimeStamp,
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: dj3 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
