#############
# $Id: calibration.pm 13203 2011-05-18 10:56:49Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2008-07-08
# Last Changed On: $Date: 2011-05-18 11:56:49 +0100 (Wed, 18 May 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.0/lib/npg_pipeline/run/calibration.pm $

package npg_pipeline::run::calibration;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13203 $ =~ /(\d+)/mxs; $r; };

extends q{npg_pipeline::run};

sub _lanes {
  my ($self) = @_;
  return $self->batch->lanes();
}

has q{_is_spiked_store} => (isa => q{HashRef}, is => q{ro}, lazy_build => 1);

sub _build__is_spiked_store {
  my ($self) = @_;
  return {};
}

sub _is_spiked {
  my ($self, $lane) = @_;
  my $position = $lane->position();
  if (defined $self->_is_spiked_store()->{$position}) {
    return $self->_is_spiked_store()->{$position};
  }

######
# when $lane provides a method to tell if the lane is spiked with control, then use this.
#  if ($lane->spiked_control()) {
#    $self->_is_spiked_store()->{$position} = 1;
#  } else {
    $self->_is_spiked_store()->{$position} = 0;
#  }
  return $self->_is_spiked_store()->{$position};
}

sub _determine_spikes_and_controls {
  my ($self) = @_;
  my $control_lanes = [];
  my $spiked_lanes  = [];
  foreach my $lane (@{$self->_lanes()}) {
    my $entity_ref = ref$lane->entity();
    if ($lane->control()) {
      push @{$control_lanes}, $lane->position();
    }
    if ($self->_is_spiked($lane)) {
      push @{$spiked_lanes}, $lane->position();
    }
  }
  return { control_lanes => $control_lanes, spiked_lanes => $spiked_lanes, };
}

sub create_table {
  my ($self, $arg_refs) = @_;

  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $arg_refs->{timestamp};

  my $determined_lanes = $self->_determine_spikes_and_controls();
  my $control_lanes    = $determined_lanes->{control_lanes};
  my $spiked_lanes     = $determined_lanes->{spiked_lanes};

  my @job_ids;
  if (scalar@{$control_lanes} == 1) {
    ### do stuff where the control_lane is used to create the calibration table for all non-spiked lanes
  } elsif (scalar@{$control_lanes}) {
    #### my $control_lane_to_make_canned_cal_from
    #### foreach my $lane (@control_lanes) { if ($lane == $self->default_control_lane()) { $control_lane_to_make_canned_cal_from = $lane }}
    #### if (!$control_lane_to_make_canned_cal_from) { $control_lane_to_make_canned_cal_from = $control_lanes[0];  }
    #### do stuff where the control_lane_to_make_canned_cal_from is used to create the calibration table for all non-spiked lanes
  }
  if (scalar@{$spiked_lanes}) {
    #### foreach my $lane (@spiked_lanes) { do somthing to create calibration table foreach lane from control spikes}
  }

  push @job_ids, @{$control_lanes}; # to be removed

  return @job_ids;
}

sub apply_table {
  my ($self, $arg_refs) = @_;

  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $arg_refs->{timestamp};

  my $determined_lanes = $self->_determine_spikes_and_controls();
  my $control_lanes    = $determined_lanes->{control_lanes};
  my $spiked_lanes = {};
  foreach my $lane (@{$determined_lanes->{spiked_lanes}}) {
    $spiked_lanes->{$lane}++;
  }

  my @job_ids;
  foreach my $lane (@{$self->_lanes()}) {
    my $position = $lane->position();
    if ($spiked_lanes->{$position}) {
      ##### do stuff which applies the lanes own spike calibration table
    } elsif (scalar@{$control_lanes}) {
      ##### do stuff which applies the control lane calibration table
    } else {
      ##### do stuff which applies the canned calibration table
    }
    push @job_ids, $position; # to be removed
  }
  return @job_ids;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::run::calibration

=head1 VERSION

$LastChangedRevision: 13203 $

=head1 SYNOPSIS

  my $oRunCalibration = npg_pipeline::run::calibration->new({
    id_run => $iIdRun,
  });

=head1 DESCRIPTION

creates run calibration table(s)

=head1 SUBROUTINES/METHODS

=head2 create_table - handler method to call to determine calibration tables to generate, and generate them

  my @job_ids;
  eval { @job_ids = $oRunCalibration->create_table(); } or do { # your error handling here };

=head2 apply_table - handler method to call to determine which calibration table to use, and run command

  my @job_ids;
  eval { @job_ids = $oRunCalibration->apply_table(); } or do { # your error handling here };

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item npg_pipeline::run

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
