#############
# $Id: link.pm 13203 2011-05-18 10:56:49Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-07-08
# Last Changed On: $Date: 2011-05-18 11:56:49 +0100 (Wed, 18 May 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.0/lib/npg_pipeline/run/folder/link.pm $

package npg_pipeline::run::folder::link;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13203 $ =~ /(\d+)/mxs; $r; };

extends q{npg_pipeline::run::folder};

Readonly::Scalar our $RECALIBRATED_TARGET_PRELIM  => q{Data/Intensities/Bustard*/GERALD*};
Readonly::Scalar our $RTA_INTENSITIES_PATH        => q{Data/Intensities};
Readonly::Scalar our $RECALIBRATED_MATCH          => q{GERALD*};
Readonly::Scalar our $MAKE_LINK_SCRIPT            => q{create_summary_link};

has q{analysis_type} => (isa => q{Str}, is => q{ro}, required => 1, writer => q{_set_analysis_type});

sub make_link {

  my ($self) = @_;

  my $rf_path     = $self->runfolder_path();

  my $recalibrated_path;
  eval {
    $recalibrated_path = $self->recalibrated_path();
  } or do {
    carp $EVAL_ERROR;
  };

  my $link  = $self->analysis_path                                      ? $self->analysis_path()
            : lc$self->analysis_type() eq q{full} && $recalibrated_path ? $recalibrated_path
            :                                                             $RECALIBRATED_TARGET_PRELIM
            ;

  my $cur_link;
  my $summary_link = $self->summary_link();
  if ( -l $summary_link) {
    $cur_link = readlink qq{$rf_path/$summary_link};
  }

  if ($cur_link and $link =~ /\G$cur_link\E/xms) {
    $self->log(qq{$summary_link link ($cur_link) already points to $link -- not changed.});
  } else {

    # Because Latest_Summary points to a directory, ln -fs gets
    # confused, so we rm it first.
    $link =~ s{\A\G$rf_path\E/}{}xms;
    my $command = qq{cd $rf_path; rm -f $summary_link; ln -fs $link $summary_link};
    $self->log(qq{Running $command});
    my $rc = `$command`;
    if ($CHILD_ERROR != 0) {
      $self->log(qq{link command failed - $EVAL_ERROR -  Error code : $CHILD_ERROR});
      croak qq{link command failed - $EVAL_ERROR - Error code : $CHILD_ERROR};
    }

  }

  return;

}

###############
# responsible for generating the bsub command to be executed
sub _generate_bsub_command {
  my ($self, $arg_refs) = @_;
  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $self->timestamp();

  my $run_folder = $self->run_folder();
  if ( !$self->folder() ) {
    $self->_set_folder( q{analysis} );
  }
  my $folder = $self->folder();

  my $command_line_options = $self->attributes_as_command_options( {
    excluded_attributes => [ qw{
      log_file_name log_file_path log_file timestamp subpath ARGV time timestamp
    } ]
  } );

  my $inst_dir = $self->_get_instrument_dir();
  $inst_dir =~ s/incoming/$folder/xms;

  my $job_name = q{create_latest_summary_link_} . $run_folder . q{_} . $timestamp;

  my $bsub_command = qq{bsub -P pipeline $required_job_completion -J $job_name };
  $bsub_command .= q{ } . ( $self->fs_resource_string( {
    counter_slots_per_job => 1,
  } ) );
  $bsub_command .=  q{ -o } . $inst_dir . q{/} . $run_folder . q{/} . qq{$job_name.out };
  $bsub_command .=  q{'} . $self->internal_script_bin() . q{/} . $MAKE_LINK_SCRIPT . q{ };
  $bsub_command .= qq{$command_line_options'};

  if ($self->verbose()) {
    $self->log($bsub_command);
  }

  return $bsub_command;
}

sub submit_create_link {
  my ($self, $arg_refs) = @_;
  my $cmd = $self->_generate_bsub_command($arg_refs);
  my $job_id = $self->submit_bsub_command($cmd);
  return $job_id;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::run::folder::link

=head1 VERSION

$LastChangedRevision: 13203 $

=head1 SYNOPSIS

  my $rfl = npg_pipeline::run::folder::link->new({
    analysis_type => <full/single>,
    run_folder    => <run_folder>,
    folder        => <analysis/outgoing>, # defaults to analysis
    analysis_path => q{Data/Intensities/Bustard_dir/GERALD_dir}, # required if you want to override any existing link
  });

=head1 DESCRIPTION

Class to create a LatestSummary link to the GERALD folder with the latest summary

=head1 SUBROUTINES/METHODS

=head2 make_link - method to call to make the link to the Latest Summary

  eval { 
    my $arg_refs = {
      required_job_completion => $sJobDependencies,
      timestamp               => $sTimeStamp,
    };
    $rfl->make_link($arg_refs);
  } or do { croak $EVAL_ERROR; };

=head2 submit_create_link - method which generates and submits an LSF job which uses dependencies such that the link will only be generated once the correct folder to point ot is there

  my $job_id;
  eval { 
    my $arg_refs = {
      required_job_completion => $sJobDependencies,
      timestamp               => $sTimeStamp,
    };
    $job_id = $rfl->submit_create_link($arg_refs);
  } or do { croak $EVAL_ERROR; };

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Moose

=item Readonly

=item npg_pipeline::run::folder

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
