#############
# $Id: move.pm 13203 2011-05-18 10:56:49Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-07-08
# Last Changed On: $Date: 2011-05-18 11:56:49 +0100 (Wed, 18 May 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.0/lib/npg_pipeline/run/folder/move.pm $

package npg_pipeline::run::folder::move;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13203 $ =~ /(\d+)/mxs; $r; };

extends q{npg_pipeline::run::folder};

Readonly::Scalar our $MOVE_FOLDER_SCRIPT  => q{move_run_folder};

sub move_runfolder {
  my ($self) = @_;

  my @runs = $self->_get_runs_for_this_runfolder();

  if ( scalar @runs == 2 ) {
    $self->log( $self->id_run() . q{ appears to be a paired run. There is another runfolder to move, but I'm not doing it!} );
  }

  eval {
    $self->_move_individual_folder( $self->name() );
    1;
  } or do {
    croak $EVAL_ERROR;
  };

  return 1;
}

sub _get_runs_for_this_runfolder {
  my ($self) = @_;
  my @runs = ($self->run());
  if($self->run()->is_paired_run() && $self->run()->run_pair()) {
    push @runs, $self->run()->run_pair();
  }
  return @runs;
}

sub _move_individual_folder {
  my ( $self, $run_name ) = @_;
  $self->log( qq{Run name: $run_name.} );

  my $folder = $self->folder();

  my $start_dir = my $end_dir = $self->runfolder_path();

  if ( ! -d $start_dir ) {
    croak qq{Failed to find $start_dir to move};
  }

  if ( ! $folder || $folder eq q{outgoing} ) {  # moves from analysis to outgoing

    $end_dir  =~ s/incoming/outgoing/xms; # most likely that this is going to say
    $end_dir  =~ s/analysis/outgoing/xms; # analysis, but just to be sure

  } else {                                   # moves from incoming to analysis

    $end_dir  =~ s/incoming/analysis/xms; # a move to analysis would mean that it should be in incoming
  }

  if ( -d $end_dir ) {
    croak qq{$end_dir already exists};
  }

  my @path = split m{/}xms, $end_dir;
  pop@path;
  $end_dir = join q{/}, @path;

  my $cmd  = qq{mv $start_dir $end_dir/};
  $self->log( qq{Move command = $cmd} );
  my $rc = qx/$cmd/;
  if ( $CHILD_ERROR != 0 ) {
    $self->log( qq{runfolder move failed - Error code: $CHILD_ERROR} );
    croak qq{runfolder move failed - Error code: $CHILD_ERROR};
  }

  $self->log( q{runfolder move success} );

  return 1;
}

###############
# responsible for generating the bsub command to be executed
sub _generate_bsub_command {
  my ($self, $arg_refs) = @_;
  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $self->timestamp();

  my $run_folder = $self->run_folder();
  if ( ! $self->folder() ) {
    $self->_set_folder( q{outgoing} );
  }
  my $folder = $self->folder();

  my $command_line_options = $self->attributes_as_command_options( {
    excluded_attributes => [ qw{
      log_file_name log_file_path log_file timestamp subpath ARGV time timestamp
    } ]
  } );

  my $inst_dir = $self->_get_instrument_dir( $self->name() );

  $inst_dir =~ s/incoming/$folder/xms; ###########
  $inst_dir =~ s/analysis/$folder/xms; # make sure that the folder is what is expected
  $inst_dir =~ s/outgoing/$folder/xms; ###########

  my $job_name = $MOVE_FOLDER_SCRIPT . q{_} . $run_folder . q{_} . $timestamp;

  my $bsub_command = qq{bsub $required_job_completion -J $job_name -q } . $self->small_lsf_queue() . q{ };
  $bsub_command .=  ( $self->fs_resource_string( {
    counter_slots_per_job => 1,
  } ) ) . q{ };
  $bsub_command .=  q{-o } . $self->make_log_dir( $inst_dir ) . q{/} . $run_folder . q{_} . $job_name . q{.out };
  $bsub_command .=  q{'} . $self->internal_script_bin() . q{/} . $MOVE_FOLDER_SCRIPT . q{ };
  $bsub_command .= qq{$command_line_options'};
  if ( $self->verbose() ) {
    $self->log( $bsub_command );
  }
  return $bsub_command;
}

sub submit_move_run_folder {
  my ($self, $arg_refs) = @_;
  my $cmd = $self->_generate_bsub_command( $arg_refs );
  my $job_id = $self->submit_bsub_command( $cmd );
  return $job_id;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::run::folder::move

=head1 VERSION

$LastChangedRevision: 13203 $

=head1 SYNOPSIS

  my $rfm = npg_pipeline::run::folder::move->new({
    run_folder => $sRunFolder,
  });

  eval { $rfm->move_runfolder(); } or do { croak $EVAL_ERROR; };

=head1 DESCRIPTION

Class which can move the run_folder from analysis to outgoing. It can do this directly,
or submit a job to LSF so that it can be done at an appropriate time.

=head1 SUBROUTINES/METHODS

=head2 move_runfolder - handler for moving the runfolder from analysis to outgoing

=head2 submit_move_run_folder - handler for submitting a job to do this to LSF

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item npg_pipeline::run::folder

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
