# $Id: 20-archive_file_generation-fastq_by_tag.t 13883 2011-08-10 09:55:39Z dj3 $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 6;
use Test::Exception;
use lib qw{t};
use t::util;

use_ok('npg_pipeline::archive::file::generation::fastq_by_tag');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();

{ 
 $ENV{TEST_RUN_OBJECT} = npg::api::run->new({
    util => $util->apiutil(),
    id_run => 1234,
  });

  $util->create_multiplex_analysis();

  my $generator;

  lives_ok { $generator = npg_pipeline::archive::file::generation::fastq_by_tag->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  }); } q{no croak creating $fastq_generator object};
  isa_ok($generator, q{npg_pipeline::archive::file::generation::fastq_by_tag}, q{$generator});

  my $arg_refs = {
    position => q{`echo $LSB_JOBINDEX`},
    fastq => q{1234_`echo $LSB_JOBINDEX`_1.fastq},
    output_dir => '/tmp',
    required_job_completion => q{-w'done(123) && done(321)'},
    split_type => q{standard},
    array_string => q{[1-4,8]},
  };

  my $bsub_command = $util->drop_temp_part_from_paths( $generator->_generate_bsub_command( $arg_refs ) );
  my $expected_command = q{bsub -q test -R 'rusage[nfs_12=8]' -w'done(123) && done(321)' -J split_fastq_by_tag_1234_end1_standard_20090709-123456[1-4,8] -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/split_fastq_by_tag_1234_end1_standard_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/split_fastq_by_tag --fastq 1234_`echo $LSB_JOBINDEX`_1.fastq --output_dir /tmp'};

  is( $bsub_command, $expected_command, q{generated bsub command is correct} );

  my $runfolder = $generator->runfolder_path();
  `cp t/data/tag_files/lane_7.tag $runfolder`; 
  `cp t/data/tag_files/lane_7.tag $runfolder/lane_1.tag`; 

  $ENV{TEST_API_UTIL} = $util->apiutil();
  my @jids;
  lives_ok { @jids = $generator->generate( $arg_refs ); } q{no croak running generate};
 
  is(scalar@jids, 6, q{6 jobs ids returned});
}

1;
