# $Id: 20-archive_file_generation-touchCompletedLane.t 13481 2011-06-29 15:16:18Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 9;
use Test::Exception;
use lib qw{t};
use t::util;

use_ok('npg_pipeline::archive::file::generation::touchCompletedLane');

my $util = t::util->new();
my $conf_path = $util->conf_path();
my $runfolder = $util->default_runfolder();

{

  $util->create_analysis();

  my $object;
  lives_ok {
    $object = npg_pipeline::archive::file::generation::touchCompletedLane->new( {
      run_folder => $runfolder,
      recalibrated_path => $util->standard_analysis_recalibrated_path(),
      lanes => [3],
    } );
  } q{created object ok};

  lives_ok {
    $object->touch_file();
  } q{runs ok};

  ok( -e $util->standard_analysis_recalibrated_path() . q{/archive/completed_lane_3}, q{file touched} );

}

{
  my $object = npg_pipeline::archive::file::generation::touchCompletedLane->new( {
    run_folder => $runfolder,
    archive_path => q{/this/should/never/ever/ever/exist/as/a/directory},
    lanes => [3],
  } );

  throws_ok {
    $object->touch_file();
  } qr{unable[ ]to[ ]touch[ ]completed_lane_3}, q{file not touched};
}

{
  
  my $object;
  lives_ok {
    $object = npg_pipeline::archive::file::generation::touchCompletedLane->new( {
      run_folder => $runfolder,
      recalibrated_path => $util->standard_analysis_recalibrated_path(),
      conf_path => $conf_path,
      domain => q{test},
      lanes => [3],
      timestamp => q{20110628-111814},
      no_bsub => 1,
    } );
  } q{created object ok};

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
    lane => 3,
  };
  my $bsub_command = $util->drop_temp_part_from_paths( $object->_generate_bsub_command($arg_refs) );

  my $expected_command = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J touch_secondary_completed_file_1234_3_20110628-111814 -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/touch_secondary_completed_file_1234_3_20110628-111814.%I.%J.out 't/bin/software/solexa/bin/npg_pipeline_touch_completed_lane --archive_path /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --conf_path data/config_files --domain test --id_run 1234 --internal_script_bin t/bin/software/solexa/bin --name IL2_1234 --no_bsub  --recalibrated_path /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto --run_folder 123456_IL2_1234 --lanes 3'};

  is( $bsub_command, $expected_command, q{command is correct} );

  delete $arg_refs->{lane};

  my @job_ids;
  lives_ok {
    @job_ids = $object->launch_bsub_job( $arg_refs );
  } q{launch_bsub_job runs ok};

  is( scalar @job_ids, 1, q{just one job id created} );

}

1;